# -*- coding: utf-8 -*-
# trimends4php.py - Removes whitespace after last ?> in PHP files.
#
# Copyright (C) 2007 - Marcin Konicki
#     based on modelines.py by Steve Frécinaux
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os
import gedit
import popen2

do_syntax_check = 1
notifications = 0
try:
	import pynotify
	if pynotify.init("Gedit trimends4php plugin"):
		notifications = 1
	else:
		print "there was a problem initializing the pynotify module"
except:
	print "you don't seem to have pynotify installed"


class Trimends4php(gedit.Plugin):
	def __init__(self):
		gedit.Plugin.__init__(self)

	def connect_handlers(self, view):
		doc = view.get_buffer()
		saved_id  = doc.connect("saved", apply_trimends4php, view)
		doc.set_data("Trimends4phpPluginHandlerIds", saved_id)
	
	def disconnect_handlers(self, view):
		doc = view.get_buffer()
		loaded_id, saved_id = doc.get_data("Trimends4phpPluginHandlerIds")
		doc.disconnect(loaded_id)
		doc.disconnect(saved_id)
		doc.set_data("Trimends4phpPluginHandlerIds", None)

	def activate(self, window):
		for view in window.get_views(): 
			self.connect_handlers(view)
			apply_trimends4php(view.get_buffer(), None, view)
			
		tab_added_id = window.connect("tab_added",
		                              lambda w, t: self.connect_handlers(t.get_view()))
		window.set_data("Trimends4phpPluginHandlerId", tab_added_id)

	def deactivate(self, window):
		tab_added_id = window.get_data("Trimends4phpPluginHandlerId")
		window.disconnect(tab_added_id)
		window.set_data("Trimends4phpPluginHandlerId", None)
		
		for view in window.get_views():
			self.disconnect_handlers(view)

def trimends4php_error(title, message):
	if (notifications == 1):
		n = pynotify.Notification(title, message, "error")
		n.set_urgency(pynotify.URGENCY_CRITICAL) # can be LOW, NORMAL or CRITICAL
		n.show()
	else:
		print title + ":\n" + message + "\n"

def apply_trimends4php(doc, error, view):
	if (error is None and doc.get_mime_type() == 'application/x-php' and doc.get_uri().find('file://') == 0):
		stats = os.stat(doc.get_uri_for_display())

		try:
			f = open(doc.get_uri_for_display(), 'r+')
			content = f.read()
			f.close()
			l = content.rsplit('?>', 1)
			if (len(l) > 1 and len(l[1].rstrip()) == 0):
				try:
					f = open(doc.get_uri_for_display(), 'w')
					f.write(l[0])
					f.write('?>')
					f.close()
				except:
					trimends4php_error("Trimming PHP file failed", "Could not write file: %s\n" % doc.get_uri_for_display())
					return
		except:
			trimends4php_error("Trimming PHP file failed", "Could not open file: %s\n" % doc.get_uri_for_display())
			return

		if (do_syntax_check == 1):
			out, inp = popen2.popen4('php -l '+doc.get_uri_for_display())
			inp.close()
			info = out.read()
			out.close()
			needle = "Parse error:"
			if (needle in info):
				info = info[info.find(needle):]
				info = info[0:info.find("\n")]
				trimends4php_error("PHP syntax error", info)

		# use utime to bring back times from before our modification, so GEdit will not annoy us with file modification warnings
		# this seems to not work on DOS partition (chown is different because of lack of owner id)?
		os.utime(doc.get_uri_for_display(),(stats.st_atime,stats.st_mtime))

